<?php

use Axepta\Payment\Util;
use JsonSchema\Validator;

/**
 * Class ControllerExtensionPaymentAxepta
 */
class ControllerExtensionPaymentAxepta extends Controller
{
	private $error = [];

	/**
	 * view admin panel
	 */
	public function index()
	{
		$this->language->load('extension/payment/axepta');
		$this->document->setTitle($this->language->get('heading_title'));
		$this->document->addScript('view/javascript/select2/js/select2.min.js');
		$this->document->addStyle('view/javascript/select2/css/select2.css');
		$this->load->model('setting/setting');

		$request = $this->request->post;

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		if(($this->request->server['REQUEST_METHOD'] == Util::METHOD_REQUEST_POST)
			&& $this->validate($request)
		) {

			$settings = array_merge($this->model_setting_setting->getSetting('axepta'), $request);
			$this->model_setting_setting->editSetting('axepta', $settings);
			$this->session->data['success'] = $this->language->get('text_success');
			$this->response->redirect($this->url->link('extension/extension', 'token=' . $this->session->data['token'], 'SSL'));

			return;
		}

		$data = [

			'button_save'   => $this->language->get('button_save'),
			'button_cancel' => $this->language->get('button_cancel'),

			'text_edit' => $this->language->get('text_edit'),

			'text_enabled'  => $this->language->get('text_enabled'),
			'text_disabled' => $this->language->get('text_disabled'),

			'text_header_tip' => $this->language->get('text_header_tip'),

			'axepta_status_label' => $this->language->get('axepta_status_label'),
			'axepta_status'       => isset($this->request->post['axepta_status'])
				? $this->request->post['axepta_status']
				: $this->config->get('axepta_status'),

			'axepta_sandbox_label'        => $this->language->get('axepta_sandbox_label'),
			'axepta_sandbox_enabled_hint' => $this->language->get('axepta_sandbox_enabled_hint'),
			'axepta_sandbox'              => isset($this->request->post['axepta_sandbox'])
				? $this->request->post['axepta_sandbox']
				: $this->config->get('axepta_sandbox'),

			'axepta_merchant_id_label' => $this->language->get('axepta_merchant_id_label'),
			'axepta_merchant_id'       => isset($this->request->post['axepta_merchant_id'])
				? $this->request->post['axepta_merchant_id']
				: $this->config->get('axepta_merchant_id'),

			'axepta_service_id_label' => $this->language->get('axepta_service_id_label'),
			'axepta_service_id'       => isset($this->request->post['axepta_service_id'])
				? $this->request->post['axepta_service_id']
				: $this->config->get('axepta_service_id'),

			'axepta_service_key_label' => $this->language->get('axepta_service_key_label'),
			'axepta_service_key'       => isset($this->request->post['axepta_service_key'])
				? $this->request->post['axepta_service_key']
				: $this->config->get('axepta_service_key'),

			'axepta_credit_type_label' => $this->language->get('axepta_credit_type_label'),
			'axepta_credit_type'       => isset($this->request->post['axepta_credit_type'])
				? $this->request->post['axepta_credit_type']
				: $this->config->get('axepta_credit_type'),

			'axepta_sort_order_label' => $this->language->get('axepta_sort_order_label'),
			'axepta_sort_order'       => isset($this->request->post['axepta_sort_order'])
				? $this->request->post['axepta_sort_order']
				: $this->config->get('axepta_sort_order'),

			'axepta_payment_title_default' => $this->language->get('axepta_payment_title_default'),
			'axepta_payment_title_label'   => $this->language->get('axepta_payment_title_label'),
			'axepta_payment_title'         => isset($this->request->post['axepta_payment_title'])
				? $this->request->post['axepta_payment_title']
				: $this->config->get('axepta_payment_title'),

			'axepta_currency_label' => $this->language->get('axepta_currency_label'),
			'axepta_all_currencies' => Util::getSupportedCurrencies(),
			'axepta_currencies'     => isset($this->request->post['axepta_currencies'])
				? $this->request->post['axepta_currencies']
				: ($this->config->get('axepta_currencies')
					?: []),

			'text_error_validation' => $this->error,

			'text_footer_notification' => $this->language->get('text_footer_notification'),
			'text_notification_url'    => HTTPS_CATALOG . 'index.php?route=extension/payment/axepta/notification',

			'confirm_url' => $this->url->link('extension/payment/axepta', 'token=' . $this->session->data['token'], true),
			'cancel_url'  => $this->url->link('extension/extension', 'token=' . $this->session->data['token'], true),

			'content_footer'      => $this->load->controller('common/footer'),
			'content_header'      => $this->load->controller('common/header'),
			'content_column_left' => $this->load->controller('common/column_left'),
			'heading_title'       => $this->language->get('text_heading_title'),

			'breadcrumbs' => [
				[
					'text'      => $this->language->get('text_home'),
					'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => false,
				],

				[
					'text'      => $this->language->get('text_payment'),
					'href'      => $this->url->link('extension/extension', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => ' :: ',
				],

				[
					'text'      => $this->language->get('text_heading_title'),
					'href'      => $this->url->link('extension/payment/axepta', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => ' :: ',
				],
			],

		];

		$this->response->setOutput($this->load->view('extension/payment/axepta', $data));

		$current_axepta_settings = $this->model_setting_setting->getSetting('axepta');

		if(isset($current_axepta_settings['axepta_error_tmp'])) {
			unset($current_axepta_settings['axepta_error_tmp']);
			$this->model_setting_setting->editSetting('axepta', $current_axepta_settings);
		}

		if(isset($current_axepta_settings['axepta_success_tmp'])) {
			unset($current_axepta_settings['axepta_success_tmp']);
			$this->model_setting_setting->editSetting('axepta', $current_axepta_settings);
		}
	}

	/**
	 * @param array $request
	 *
	 * @return bool
	 */
	protected function validate($request)
	{
		if(!$this->user->hasPermission('modify', 'extension/payment/axepta')) {
			$this->error['error_warning_permissions'] = $this->language->get('error_permission');

			return false;
		}

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$schema = [
			'type'       => 'object',
			'properties' => [

				'axepta_status'        => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'axepta_sandbox'       => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'axepta_merchant_id'   => [
					'type' => 'string',
				],
				'axepta_service_id'    => [
					'type' => 'string',
				],
				'axepta_service_key'   => [
					'type' => 'string',
				],
				'axepta_credit_type'   => [
					'type' => 'string',
				],
				'axepta_sort_order'    => [
					'type'      => 'string',
					'minLength' => 1,
				],
				'axepta_payment_title' => [
					'type' => 'string',
				],
				'axepta_currencies'    => [
					'type'        => 'array',
					'items'       => [
						'type' => 'string',
						'enum' => array_values(Util::getSupportedCurrencies()),
					],
					'minItems'    => 1,
					'uniqueItems' => true,
				],
			],

			'required'             => [
				'axepta_status',
				'axepta_sandbox',
				'axepta_merchant_id',
				'axepta_service_id',
				'axepta_service_key',
				'axepta_sort_order',
				'axepta_payment_title',
				'axepta_currencies',
			],
			'additionalProperties' => false,
		];

		$data = json_decode(json_encode($request));

		$validator = new Validator();
		$validator->validate($data, $schema);

		$this->error = [];

		foreach($validator->getErrors() as $key => $value) {

			if($value['constraint'] === 'additionalProp') {
				$this->error['error_additional_properties'] = $this->language->get('error_additional_properties');
			} else {

				$this->error['error_' . $value['property']] = $this->language->get('error_validation');
			}
		}

		return empty($this->error);
	}
}
