<?php

use Axepta\Payment\CartData;
use Axepta\Payment\Notification;
use Axepta\Payment\Paywall;
use Axepta\Payment\Util;

/**
 * Class ControllerExtensionPaymentAxepta
 */
class ControllerExtensionPaymentAxepta extends Controller
{

	/**
	 * plugin version
	 */
	const VERSION = '1.0.1';

	/**
	 * @return mixed
	 */
	public function index()
	{

		$this->language->load('extension/payment/axepta');
		$this->load->model('setting/setting');

		$data = [
			'url_pay'         => $this->url->link('extension/payment/axepta/pay', '', true),
			'pay_with_axepta' => $this->language->get('pay_with_axepta'),
			'error_pay'       => $this->language->get('error_pay'),
		];

		return $this->load->view('extension/payment/axepta', $data);
	}

	/**
	 * Initiate form payment, make payment
	 */
	public function pay()
	{

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$this->load->model('checkout/order');
		$this->language->load('extension/payment/axepta');
		$this->load->model('setting/setting');

		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		$paywall = new Paywall(
			($this->config->get('axepta_sandbox') == 1
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION),
			$this->config->get('axepta_service_key'),
			$this->config->get('axepta_service_id'),
			$this->config->get('axepta_merchant_id')
		);

		$cart_data = $this->getCart($order_info);

		$order_data = Paywall::prepareData(
			Util::convertAmountToFractional($order_info['total'] * $order_info['currency_value']),
			strtoupper($order_info['currency_code']),
			$order_info['order_id'],
			$order_info['payment_firstname'],
			$order_info['payment_lastname'],
			$order_info['email'],
			$order_info['telephone'],
			$this->url->link('checkout/success'),
			$this->url->link('checkout/success'),
			$this->url->link('checkout/success'),
			$this->config->get('axepta_credit_type'),
			$cart_data,
			self::VERSION . ';opencart_' . VERSION
		);

		$data = [
			'form' => $paywall->buildOrderForm(
				$order_data,
				$this->language->get('submit')
			),

			'text_tip_checkout' => $this->language->get('text_tip_checkout'),
		];

		$this->model_checkout_order->addOrderHistory($order_info['order_id'], 01);

		$this->load->language('checkout/cart');
		$this->cart->clear();

		unset($this->session->data['vouchers']);
		unset($this->session->data['shipping_method']);
		unset($this->session->data['shipping_methods']);
		unset($this->session->data['payment_method']);
		unset($this->session->data['payment_methods']);
		unset($this->session->data['reward']);

		$this->response->setOutput($this->load->view('extension/payment/axepta_redirect', $data));
	}

	/**
	 * Support incoming notification (and validate)
	 *
	 * @throws Exception
	 */
	public function notification()
	{

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$notification = new Notification(
			$this->config->get('axepta_service_id'),
			$this->config->get('axepta_service_key')
		);

		$result_check_request_notification = $notification->checkRequest();

		if(is_int($result_check_request_notification)) {
			echo Notification::formatResponse(
				Notification::NS_ERROR,
				$result_check_request_notification
			);
			exit();
		}

		$this->load->model('checkout/order');

		$order = $this->model_checkout_order->getOrder($result_check_request_notification['payment']['orderId']);

		if(!$order) {
			echo Notification::formatResponse(
				Notification::NS_ERROR,
				Notification::NC_ORDER_NOT_FOUND
			);
			exit();
		}

		if(!Notification::checkRequestAmount(
			$result_check_request_notification,
			Util::convertAmountToFractional($order['total'] * $order['currency_value']),
			strtoupper($order['currency_code']))
		) {

			echo Notification::formatResponse(
				Notification::NS_ERROR,
				Notification::NC_AMOUNT_NOT_MATCH
			);
			exit();
		}

		$orderStatusBefore = $order['order_status_id'];
		$orderStatusAfter = null;

		if($order['order_status_id'] === '1' || $order['order_status_id'] === '10') {

			switch($result_check_request_notification['payment']['status']) {
				case Notification::PS_SETTLED:
					$orderStatusAfter = 02;
					$this->model_checkout_order->addOrderHistory($order['order_id'], $orderStatusAfter);
					break;
				case Notification::PS_REJECTED:
					$orderStatusAfter = 10;
					$this->model_checkout_order->addOrderHistory($order['order_id'], $orderStatusAfter);
					break;
				default:
					echo Notification::formatResponse(
						Notification::NS_OK,
						Notification::NC_ORDER_STATUS_NOT_CHANGED,
						$orderStatusBefore
					);
					exit();
			}
		}

		echo Notification::formatResponse(
			Notification::NS_OK,
			Notification::NC_OK,
			$orderStatusBefore,
			$orderStatusAfter
		);
	}

	/**
	 * @param array $order
	 *
	 * @return array
	 */
	private function getAddressDelivery($order)
	{

		// check if shipping data is provided - 'shipping_city' is enough
		if($order['shipping_city']) {
			return [
				'shipping_city'       => $order['shipping_city'],
				'shipping_name'       => $order['shipping_firstname'] . ' ' . $order['shipping_lastname'],
				'telephone'           => $order['telephone'],
				'shipping_address'    => isset($order['shipping_address_2'])
					? $order['shipping_address_1'] . ' ' . $order['shipping_address_2']
					: $order['shipping_address_1'],
				'shipping_iso_code_2' => $order['shipping_iso_code_2'],
				'shipping_postcode'   => $order['shipping_postcode'],
			];
		} else {
			return [
				'shipping_city'       => $order['payment_city'],
				'shipping_name'       => $order['payment_firstname'] . ' ' . $order['payment_lastname'],
				'telephone'           => $order['telephone'],
				'shipping_address'    => isset($order['payment_address_2'])
					? $order['payment_address_1'] . ' ' . $order['payment_address_2']
					: $order['payment_address_1'],
				'shipping_iso_code_2' => $order['payment_iso_code_2'],
				'shipping_postcode'   => $order['payment_postcode'],
			];
		}
	}

	/**
	 * @param array $order
	 *
	 * @return string
	 */
	private function getCart($order)
	{

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$this->load->model('catalog/product');

		$cart_data = new CartData();

		$cart_data->setGrandTotal(Util::convertAmountToFractional($order['total'] * $order['currency_value']));

		foreach($this->cart->getProducts() as $cart_item) {

			$price = $this->tax->calculate(
				$cart_item['price'],
				$cart_item['tax_class_id'],
				$this->config->get('config_tax'));

			if(isset($cart_item['special']) && $cart_item['special']) {
				$price = $this->tax->calculate(
					$cart_item['special'],
					$cart_item['tax_class_id'],
					$this->config->get('config_tax'));
			}

			$cart_data->addItem(
				$cart_item['name'],
				Util::convertAmountToFractional($price * $order['currency_value']),
				(float) $cart_item['quantity']
			);
		}

		$cart_data->setAddressBilling(
			$order['payment_city'],
			isset($order['payment_address_2'])
				? $order['payment_address_1'] . ' ' . $order['payment_address_2']
				: $order['payment_address_1'],
			$order['payment_iso_code_2'],
			$order['payment_postcode']

		);

		$addressDelivery = $this->getAddressDelivery($order);

		$cart_data->setAddressDelivery(
			$addressDelivery['shipping_city'],
			$addressDelivery['shipping_address'],
			$addressDelivery['shipping_iso_code_2'],
			$addressDelivery['shipping_postcode']
		);

		$this->load->model('extension/total/coupon');

		return $cart_data->prepareCartData();
	}
}
